/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojo.java 16:56:43 ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.plugin.jbiplugin;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.Set;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.resolver.ArtifactNotFoundException;
import org.apache.maven.artifact.resolver.ArtifactResolutionException;
import org.apache.maven.plugin.MojoExecutionException;
import org.objectweb.petals.jbi.descriptor.ComponentDescription;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jmx.api.PetalsComponentComponentClient;
import org.objectweb.petals.tools.jmx.api.PetalsDeploymentServiceClient;
import org.objectweb.petals.tools.jmx.api.PetalsInstallationServiceClient;
import org.objectweb.petals.tools.jmx.api.PetalsInstallerComponentClient;
import org.objectweb.petals.tools.jmx.api.PetalsJMXClient;
import org.objectweb.petals.tools.jmx.api.exception.AttributeErrorException;
import org.objectweb.petals.tools.jmx.api.exception.ComponentComponentDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.ComponentErrorException;
import org.objectweb.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.objectweb.petals.tools.jmx.api.exception.DeploymentServiceDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.DeploymentServiceErrorException;
import org.objectweb.petals.tools.jmx.api.exception.InstallationServiceDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.InstallationServiceErrorException;
import org.objectweb.petals.tools.jmx.api.exception.InstallerComponentDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.PerformActionErrorException;
import org.objectweb.petals.tools.jmx.api.exception.ServiceAssemblyStartedException;

/**
 * Install a JBI Archive
 * 
 * @goal jbi-install
 * @description Install a JBI Archive
 * 
 * @author cdeneux - Capgemini Sud
 */
public class JBIInstallMojo extends JBIAbstractJMXMojo {

	/**
	 * Flag to force the uninstallation, if needed.
	 * 
	 * @parameter expression="false"
	 * @editable true
	 * @description Flag to force the uninstallation, if needed.
	 */
	protected Boolean force;

	/**
	 * When installing a service assembly, if some component are missing or not
	 * started, they can be installed from Maven repositories and/or started if
	 * this flag value is <code>true</code>. This paramter is mainly used to
	 * install ans start the components that are not in the Maven reactor. A
	 * component inside the Maven reactor will be installed/started before its
	 * service assemblies by the ordering of Maven dependency mechanism.
	 * 
	 * @parameter expression="false"
	 * @editable true
	 * @description When installing a service assembly, if some component are
	 *              missing or not started, they can be installed from Maven
	 *              repositories and/or started if this flag value is
	 *              <code>true</code>. This paramter is mainly used to
	 *              install ans start the components that are not in the Maven
	 *              reactor. A component inside the Maven reactor will be
	 *              installed/started before its service assemblies by the
	 *              ordering of Maven dependency mechanism.
	 */
	protected Boolean startMissingComponents;

	/**
	 * Execute the plugin
	 */
	public void execute() throws MojoExecutionException {

		try {
			this.readJbiDescriptor();

			this.info("Using JMX Petals server located at: " + this.host + ":"
					+ this.port);

			PetalsJMXClient petalsJMXClient = new PetalsJMXClient(this.host,
					this.port, this.username, this.password);

			if (PACKAGING_COMPONENT.equals(project.getPackaging())) {
				this.installComponent(petalsJMXClient);
			} else if (PACKAGING_SU.equals(project.getPackaging())) {
				/*
				 * NOP because an SU is not installable, see its service
				 * assembly.
				 * 
				 * No error is returned to be able to use the same plugin
				 * configuration for all JBI artefacts.
				 */
			} else if (PACKAGING_SA.equals(project.getPackaging())) {
				this.installServiceAssembly(petalsJMXClient);
			} else if (PACKAGING_SL.equals(project.getPackaging())) {
				this.installSharedLibrary(petalsJMXClient);
			}
		} catch (ConnectionErrorException e) {
			throw new MojoExecutionException("Error", e);
		}

	}

	private void installComponent(PetalsJMXClient petalsJMXClient)
			throws MojoExecutionException {

		try {
			String componentName = this.getComponentNameFromJBIDescriptor();

			URL jbiPackageURL = new URL("file:///"
					+ new File(outputDirectory + File.separator + jbiName
							+ ".zip").getAbsolutePath());

			installComponentCore(petalsJMXClient, componentName, jbiPackageURL,
					force.booleanValue());

		} catch (MalformedURLException e) {
			throw new MojoExecutionException(
					"Invalid URL for JBI archive location", e);
		}

	}

	private void installComponentCore(PetalsJMXClient petalsJMXClient,
			String componentName, URL jbiPackageURL, boolean forceInstallation)
			throws MojoExecutionException {

		try {

			boolean componentInstalled = true;
			boolean componentInstallerLoaded = true;

			PetalsComponentComponentClient pccc = null;
			PetalsInstallerComponentClient picc = null;
			PetalsInstallationServiceClient pisc = petalsJMXClient
					.getInstallationServiceClient();

			// We check if the component and its installer exist
			try {
				pccc = petalsJMXClient
						.getComponentComponentClient(componentName);
			} catch (ComponentComponentDoesNotExistException e1) {
				// The component is not installed
				this.info("The component is not installed.");
				componentInstalled = false;
			}
			try {
				picc = petalsJMXClient
						.getInstallerComponentClient(componentName);
			} catch (InstallerComponentDoesNotExistException e1) {
				// The component installer is not loaded
				this.info("The component installer is not loaded.");
				componentInstallerLoaded = false;
			}

			if (!forceInstallation && componentInstalled) {
				throw new MojoExecutionException(
						"The component is already installed");
			} else if (!forceInstallation && componentInstallerLoaded) {
				throw new MojoExecutionException(
						"The component installer is already loaded");
			} else if (forceInstallation) {
				// We force the installation
				this.info("The force mode is asked.");

				if (componentInstalled) {
					// We force the shutdown and installation of the component:
					if (PetalsComponentComponentClient.State.STARTED
							.equals(pccc.getState())) {
						// We stop the component
						this.info("Stoping the component.");

						try {
							pccc.stop(true);
						} catch (ServiceAssemblyStartedException e) {
							throw new MojoExecutionException(
									"A service assembly is not stop", e);
						}
					}
					if (PetalsComponentComponentClient.State.STOPPED
							.equals(pccc.getState())) {
						// We shutdown the component
						this.info("Shutdowning the component.");
						pccc.shutdown();
					}
					if (!PetalsComponentComponentClient.State.SHUTDOWN
							.equals(pccc.getState())) {
						// The component state is not "SHUTDOWN", so a problem
						// occured during the component shutdowning.

						throw new MojoExecutionException(
								"The component is not in the expected state (SHUTDOWN)");
					}
					this.info("The component is shutdowned.");

					if (picc.isInstalled()) {
						this.info("Uninstalling the component.");
						picc.uninstall();
					}

					this.info("The component is not installed.");
				}

				// At this point the component is not installed

				if (componentInstallerLoaded) {
					// We unload the installer
					this.info("Unloading the component installer.");
					pisc.unloadInstaller(componentName);
				}
				this.info("The component installer is not loaded.");
			}

			// At this point, we have a clean environement.
			// We can install the JBI component
			this.info("Loading the component installer ("
					+ jbiPackageURL.toExternalForm() + ").");
			picc = pisc.loadNewInstaller(jbiPackageURL);
			this.info("The component installer is loaded.");

			this.info("Installing the component.");
			picc.install();
			this.info("The component is installed.");

		} catch (ComponentErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (ConnectionErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (AttributeErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (PerformActionErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (InstallationServiceErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (InstallationServiceDoesNotExistException e) {
			throw new MojoExecutionException(
					"Unable to find the installation service ", e);
		} catch (DeploymentServiceErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (DeploymentServiceDoesNotExistException e) {
			throw new MojoExecutionException(
					"Unable to find the deployment service ", e);
		}
	}

	private void installServiceAssembly(PetalsJMXClient petalsJMXClient)
			throws MojoExecutionException {

		try {

			String serviceAssemblyName = this
					.getServiceAssemblyNameFromJBIDescriptor();

			PetalsDeploymentServiceClient pdsc = petalsJMXClient
					.getDeploymentServiceClient();

			// We check that all needed components are installed
			this.info("Checking installed components ...");
			checkAndStartComponents(petalsJMXClient);
			this.info("Installed components checked.");

			// We check if the service assembly is already deployed
			if (pdsc.isServiceAssemblyDeployed(serviceAssemblyName)) {
				// The service assembly is deployed
				this.info("The service assembly is already installed.");

				if (force) {
					// We force the deployment
				}
			}

			// At this point, we have a clean environement.
			// We can install the JBI component
			URL jbiPackageURL = new URL("file:///"
					+ new File(outputDirectory + File.separator + jbiName
							+ ".zip").getAbsolutePath());

			this.info("Deploying the service assembly.");
			String deployRes = pdsc.deploy(jbiPackageURL);
			this.info("Service assembly deployment result: " + deployRes);

		} catch (MalformedURLException e) {
			throw new MojoExecutionException(
					"Invalid URL for JBI archive location", e);
		} catch (ConnectionErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (PerformActionErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (DeploymentServiceErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (DeploymentServiceDoesNotExistException e) {
			throw new MojoExecutionException(
					"Unable to find the deployment service ", e);
		}
	}

	private void installSharedLibrary(PetalsJMXClient petalsJMXClient) {
		this
				.info("Shared Library installation skip beacuase this feature is not available now. Coming next.");
	}

	/**
	 * <p>
	 * This method checks that all components needed by a service assembly are
	 * installed and started.
	 * </p>
	 * <p>
	 * If a component is missing and the parameter <code>startMissingComponents</code>
	 * is set to <code>true</code>, the component will be installed from the
	 * declared Maven repositories and started if needed. The parameter
	 * <code>force</code> is not used to drive the component installation
	 * processing.
	 * </p>
	 * 
	 * @param petalsJMXClient
	 *            The JMX connection to use for JMX calls.
	 * @throws MojoExecutionException
	 */
	private void checkAndStartComponents(PetalsJMXClient petalsJMXClient)
			throws MojoExecutionException {
		try {
			List<String> usedComponents = this
					.getComponentsFromServiceAssemblyFromJBIDescriptor();
			for (String usedComponent : usedComponents) {
				PetalsComponentComponentClient pccc = null;

				try {
					this.info("\tChecking " + usedComponent + " ...");
					pccc = petalsJMXClient
							.getComponentComponentClient(usedComponent);
					this.info("\t" + usedComponent + " installed.");

				} catch (ComponentComponentDoesNotExistException e) {
					// The component is not installed
					this.info("\t" + usedComponent + " not installed.");
					if (startMissingComponents.booleanValue()) {
						// We install it from Maven repositories
						pccc = installComponentFromMaven(petalsJMXClient,
								usedComponent);
						if (pccc == null) {
							throw new MojoExecutionException(
									"Unable to install the dependent component.");
						}
					}
				}

				if (pccc != null) {
					// The component is installed
					if ((PetalsComponentComponentClient.State.STOPPED
							.equals(pccc.getState()))
							|| (PetalsComponentComponentClient.State.SHUTDOWN
									.equals(pccc.getState()))) {
						this.info("\tStarting " + usedComponent + "...");
						pccc.start();
						this.info("\t" + usedComponent + " started.");
					}
				}
			}
		} catch (ComponentErrorException e) {
			throw new MojoExecutionException("Error:", e);
		} catch (ConnectionErrorException e) {
			throw new MojoExecutionException("Error:", e);
		} catch (PerformActionErrorException e) {
			throw new MojoExecutionException("Error on starting component:", e);
		} catch (AttributeErrorException e) {
			throw new MojoExecutionException(
					"Error getting the component status:", e);
		}
	}

	/**
	 * 
	 * Install a component from a JBI archive located in a Maven repository.
	 * 
	 * @param petalsJMXClient
	 *            The JMX connection to use for JMX calls.
	 * @param componentName
	 *            The component identifier.
	 * @return The associated <code>PetalsComponentComponentClient</code> if
	 *         the installation is successfulled, <code>null</code> otherwise.
	 * @throws MojoExecutionException
	 */
	private PetalsComponentComponentClient installComponentFromMaven(
			PetalsJMXClient petalsJMXClient, String componentName)
			throws MojoExecutionException {

		try {

			PetalsComponentComponentClient pccc = null;

			this.info("\t\tInstalling " + componentName
					+ " from a Maven repository...");
			boolean componentInstalled = false;

			Set<Artifact> dependencies = project.getDependencyArtifacts();
			for (Artifact dependency : dependencies) {
				if (PACKAGING_COMPONENT.equals(dependency.getType())) {

					try {
						this.artifactResolver.resolve(dependency, this.project
								.getRemoteArtifactRepositories(),
								this.localRepository);

						// We check if the artifact is the expected component
						// using its jbi.xml
						JBIDescriptor jbiComponentDescriptor = this
								.readJbiDescriptor(dependency.getFile());
						ComponentDescription componentDescription = jbiComponentDescriptor
								.getComponent();

						if (componentName.equals(componentDescription
								.getIdentification().getName())) {
							// We install the component
							this.installComponentCore(petalsJMXClient,
									componentName,
									dependency.getFile().toURL(), false);
							componentInstalled = true;
						}
					} catch (ArtifactResolutionException e) {
						this.error("\t\tError during the artifact resolution.",
								e);
					} catch (ArtifactNotFoundException e) {
						this.error("\t\tArtifact not found.");
					}
				}
			}

			if (componentInstalled) {
				this.info("\t\t" + componentName + " installed.");
				pccc = petalsJMXClient
						.getComponentComponentClient(componentName);
			} else {
				this.info("\t\t" + componentName + " not installed.");
			}

			return pccc;

		} catch (MalformedURLException e) {
			throw new MojoExecutionException(
					"Invalid URL for JBI archive location", e);
		} catch (ConnectionErrorException e) {
			throw new MojoExecutionException(
					"Unbale to retrieve the JMX component associated to the JBI component",
					e);
		} catch (ComponentErrorException e) {
			throw new MojoExecutionException(
					"Unbale to retrieve the JMX component associated to the JBI component",
					e);
		} catch (ComponentComponentDoesNotExistException e) {
			throw new MojoExecutionException(
					"Unbale to retrieve the JMX component associated to the JBI component",
					e);
		}

	}
}
