/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojo.java 16:56:43 ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.plugin.jbiplugin;

import org.apache.maven.plugin.MojoExecutionException;
import org.objectweb.petals.tools.jmx.api.PetalsComponentComponentClient;
import org.objectweb.petals.tools.jmx.api.PetalsJMXClient;
import org.objectweb.petals.tools.jmx.api.exception.AttributeErrorException;
import org.objectweb.petals.tools.jmx.api.exception.ComponentComponentDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.ComponentErrorException;
import org.objectweb.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.objectweb.petals.tools.jmx.api.exception.PerformActionErrorException;

/**
 * Start a JBI Artefact (Component / Service Assembly)
 * 
 * @goal jbi-start
 * @description Start a JBI Artefact (Component / Service Assembly)
 * 
 * @author cdeneux - Capgemini Sud
 */
public class JBIStartMojo extends JBIAbstractJMXMojo {

	/**
	 * Execute the plugin
	 */
	public void execute() throws MojoExecutionException {

		try {
			this.readJbiDescriptor();

			this.getLog().info(
					"Using JMX Petals server located at: " + this.host + ":"
							+ this.port);

			PetalsJMXClient petalsJMXClient = new PetalsJMXClient(this.host,
					this.port, this.username, this.password);

			if (PACKAGING_COMPONENT.equals(project.getPackaging())) {
				this.startComponent(petalsJMXClient);
			} else if (PACKAGING_SU.equals(project.getPackaging())) {
				/*
				 * NOP because an SU is not startable, see its service assembly.
				 * 
				 * No error is returned to be able to use the same plugin
				 * configuration for JBI artefacts.
				 */
			} else if (PACKAGING_SA.equals(project.getPackaging())) {
				this.startServiceAssembly(petalsJMXClient);
			} else if (PACKAGING_SL.equals(project.getPackaging())) {
				/*
				 * NOP because an SL is not startable.
				 * 
				 * No error is returned to be able to use the same plugin
				 * configuration for JBI artefacts.
				 */
			}
		} catch (ConnectionErrorException e) {
			throw new MojoExecutionException("Error", e);
		}

	}

	private void startComponent(PetalsJMXClient petalsJMXClient) throws MojoExecutionException {

		try {

			String componentName = this.getComponentNameFromJBIDescriptor();

			PetalsComponentComponentClient pccc = null;

			// We check if the component exists
			pccc = petalsJMXClient.getComponentComponentClient(componentName);

			if (PetalsComponentComponentClient.State.STARTED.equals(pccc
					.getState())) {
				// The component is already started
				throw new MojoExecutionException(
						"The component is already started.");
			}

			if ((PetalsComponentComponentClient.State.SHUTDOWN.equals(pccc
					.getState()))
					|| (PetalsComponentComponentClient.State.STOPPED
							.equals(pccc.getState()))) {
				// We shutdown the component
				if (verbose) {
					this.getLog().info("Starting the component.");
				}

				pccc.start();

				if (verbose) {
					this.getLog().info("The component is started.");
				}
			}

		} catch (ComponentComponentDoesNotExistException e1) {
			// The component is not installed
			if (verbose) {
				this.getLog().error("The component is not installed.");
			}
			throw new MojoExecutionException("The component is not installed.");
		} catch (ComponentErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (ConnectionErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (AttributeErrorException e) {
			throw new MojoExecutionException("Error", e);
		} catch (PerformActionErrorException e) {
			throw new MojoExecutionException("Error", e);
		}
	}

	private void startServiceAssembly(PetalsJMXClient petalsJMXClient) {
		this
				.getLog()
				.info(
						"Service Assembly installation skip beacuase this feature is not available now. Coming next.");
	}
}
